#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"

#
# SoundDesk (C) 1998 Donald M Burns
#
#	Version 0 - well we all have to start somewhere!
#	
#	Version 1 - a few requested changes added.
#

global mixer rc
global channels
global leftvol rightvol ganged recordsource

set mixer	"/dev/mixer"
set mixcap	"~/.mixercap"
set channels {}

# create the mixcap file if it doesn't exist, or its zero length
if {![file exists $mixcap] || ![file size $mixcap]} {
	wm iconify .
	set a [tk_messageBox -message "No start-up file found.\n\nCreate it?" -type yesno]

	if {$a == "yes"} {
		puts [set mixcapc [open $mixcap w+]] [exec mixer]
		close $mixcapc 
	} else { exit 0 }
}

#
#	Defining the procedures that run this turkey
#

# A function to actually set the mixer channel by name from the global variables
proc mixset {name} {
	global channels
	global leftvol rightvol ganged recordsource

	# Create the command and execute it.
	set newrec [exec mixer $name $leftvol($name) $rightvol($name) $recordsource($name)]

	# Now sort out what's to be shown as recording from the above
	# command's return value.

	# turn them all off and just turn on whats needed.
	foreach n $channels { set recordsource($n) normal }
	foreach n $newrec  { set recordsource($n) record }
}	

# A call-back to set ganging
proc setganged {name} {
	global leftvol rightvol ganged

	# make the right slider jump to the left's level.
	if {$ganged($name)} {
		set rightvol($name) [expr ($rightvol($name) + $leftvol($name))/2]
		set leftvol($name) $rightvol($name)
	}
	mixset $name
}

# A call-back for a mixer change
proc setlevel {name side value} {

	global leftvol rightvol ganged

	if {$ganged($name)} {
		if {$side == "left"} { set rightvol($name) $leftvol($name) }
		if {$side == "right"} { set leftvol($name) $rightvol($name) }
	}
	mixset $name
}
		
# A procedure that creates individual mixer panels
proc panel { l } {
	# the list l:- name label lvol rvol recsource polophony

	global leftvol rightvol ganged recordsource channels
	if {$l == {}} return

	set name	[lindex $l 0]
	set label	[lindex $l 1]
	set lvol	[lindex $l 2]
	set rvol	[lindex $l 3]
	set recsrc	[lindex $l 4]
	set poly	[lindex $l 5]

	# set the start-up levels and gang them together
	set leftvol($name) $lvol
	set rightvol($name) $rvol
	set ganged($name) 1

	# make a note of the channel name
	set channels [concat $channels $name]

	# make all the piece part widgets
	frame .$name -relief groove -border 2

	frame .$name.c -relief flat -height 44

	checkbutton .$name.c.rec -text Record -indicatoron 0 \
		-variable recordsource($name) -command "mixset $name" \
		-offvalue normal -onvalue record

	checkbutton .$name.c.gang -text Gang -indicatoron 0 \
		-variable ganged($name) -command "setganged $name" \
		-selectcolor grey

	frame .$name.s -relief flat 

	scale .$name.s.left  -showvalue 0 -width 10 -from 100 -to 0 \
		-tickinterval 20 -variable $name.s.left.vol \
		-variable leftvol($name) -command "setlevel $name left"

	scale .$name.s.right -showvalue 0 -width 10 -from 100 -to 0 \
		-tickinterval 0 -variable $name.s.right.vol \
		-variable rightvol($name) -command "setlevel $name right"

	entry .$name.label  -width 8 -justify center
		.$name.label insert 0 $label

	# put it together
	pack .$name.c -side top -fill x -expand 0 
	pack propagate .$name.c 0
	if {$recsrc == "source"} {
		pack .$name.c.rec -side top -fill x
	}
	if {$poly == "mono"} {
		pack .$name.s.left  -side left -fill y -expand 1
	}
	if {$poly == "stereo"} {
		pack .$name.s.left  -side left -fill y -expand 1
		pack .$name.s.right -side right -fill y -expand 1
		pack .$name.c.gang -side bottom -fill x
	}
	pack .$name.s  -fill both -expand 1 -padx 2
	pack .$name.label -side bottom -fill x -pady 2
	pack .$name -side right -fill y -anchor s -padx 1 -pady 3
}

# A routine to cleanly save and exit

proc save_and_exit {} {
	global mixer mixcap channels

	# read back the mixer specifications
	set mdat [split [read [set ch [open $mixcap r]]] "\n"]
	close $ch

	set r [open $mixcap w+]
	foreach i $mdat {
		set l [split $i ":"]
		if { $i != "" } {
			set name	[lindex $l 0]
			set label	[.${name}.label get]
			set lvol	[lindex $l 2]
			set rvol	[lindex $l 3]
			set recsrc	[lindex $l 4]
			set poly	[lindex $l 5]
			puts $r "$name:$label:$lvol:$rvol:$recsrc:$poly"
		}
	}
	close $r
	exit 0
}

# Read in the mixer mixcap file and configure the mixer.
set mdat [split [read [set ch [open $mixcap r]]] "\n"]
close $ch
foreach i $mdat {
	panel [split $i ":"]
}

# Finally, set-up the window manager and we're off!
update idletasks
wm protocol . WM_DELETE_WINDOW save_and_exit
wm title .  "SoundDesk"
set xsize [lindex [split [wm geometry .] {"x" "+"}] 0]
wm minsize . $xsize 182
wm maxsize . $xsize 500
